// ◇ UTF-8
//
// シェーダデタッチバグ再現プログラム
// 2012/08/10 Hiroaki GOTO as GORRY.
// 
// ICONIA TAB A500では、リンクが終了したシェーダプログラムに対してglDetachShader()を
// 行うと、そのシェーダプログラムの動作が不定となる。
// 当プログラムは、この現象を再現する。
// 
// 当テストコードでは、以下の動作をする。
// ■glDetachShader()が正しいとき
//   ・すべてのモードで白い円・四角・星が描かれる。
// ■ICONIA TAB A500
//   ・「Detach Shader」を実行すると描画が正常でなかったり、異常終了する。
// 
// 起動後、最初の状態ではglDetachShader()を実行していない。
// メニューから[Detach Shader]を選ぶと、glDetachShader()テストを行うことができる。

package net.gorry.example.detachbug;

import android.net.Uri;
import android.os.Bundle;
import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.content.pm.PackageManager.NameNotFoundException;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;

/**
 * @author gorry
 *
 */
public class ActivityMain extends Activity {
	private static final String TAG="DetachBug";
	private static final String SUBTAG="ActivityMain";
	private static final boolean Verbose = false;
	private static final boolean Debug = false;
	private static final boolean Info = false;
	// if (Verbose) Log.v(TAG, SUBTAG+": "+"");
	// if (Debug) Log.d(TAG, SUBTAG+": "+"");
	// if (Info) Log.i(TAG, SUBTAG+": "+"");

	private Activity me;
	private MainView mView;

	@Override
	public void onCreate(Bundle savedInstanceState) {
		if (Info) Log.i(TAG, SUBTAG+": "+"onCreate(): start");
		super.onCreate(savedInstanceState);

		me = this;
		mView = new MainView(this, 8,8,8,0, 0, 0);

		setContentView(mView);
		if (Info) Log.i(TAG, SUBTAG+": "+"onCreate(): end");
	}

	/**
	 * onResume
	 */
	@Override protected void onResume() {
		if (Info) Log.i(TAG, SUBTAG+": "+"onResume(): start");
		super.onResume();

		if (mView != null) {
			mView.onResume();
		}
		
		if (Info) Log.i(TAG, SUBTAG+": "+"onResume(): end");
	}

	/**
	 * onPause
	 */
	@Override protected void onPause() {
		if (Info) Log.i(TAG, SUBTAG+": "+"onPause(): start");

		if (mView != null) {
			mView.onPause();
		}

		super.onPause();
		if (Info) Log.i(TAG, SUBTAG+": "+"onPause(): end");
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		getMenuInflater().inflate(R.menu.activity_main, menu);
		return true;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		boolean ret = true;
		switch (item.getItemId()) {
		default:
			ret = super.onOptionsItemSelected(item);
			break;
		case R.id.menu_about:
			showAbout();
			ret = true;
			break;
		case R.id.menu_test1:
			setDetached(0);
			ret = true;
			break;
		case R.id.menu_test2:
			setDetached(1);
			ret = true;
			break;
		case R.id.menu_test3:
			setDetached(2);
			ret = true;
			break;
		case R.id.menu_test4:
			setDetached(3);
			ret = true;
			break;
		}
		return ret;
	}

	/**
	 * デタッチのON/OFFを切り替えて自分を再起動する
	 */
	public void setDetached(int mode) {
		MainView.mDetachTestMode = mode;
		final Intent intent = new Intent(getApplicationContext(), ActivityMain.class);
		me.startActivity(intent);
		me.finish();
	}

	/**
	 * Aboutを表示
	 */
	public void showAbout() {
		String versionName = null;
		final PackageManager pm = getPackageManager();
		try {
			PackageInfo info = pm.getPackageInfo(me.getPackageName(), 0);
			versionName = info.versionName;
		} catch (final NameNotFoundException e) {
			e.printStackTrace();
		}
		final AlertDialog.Builder bldr = new AlertDialog.Builder(me);
		bldr.setTitle(me.getString(R.string.app_name));
		bldr.setMessage("Version " + versionName + "\n" + me.getString(R.string.copyright));
		bldr.setPositiveButton(R.string.contact_uri_text, onContactClick);
		bldr.setIcon(R.drawable.ic_launcher);
		bldr.create().show();

	}

	private final DialogInterface.OnClickListener onContactClick = new DialogInterface.OnClickListener() {
		public void onClick(DialogInterface dialog, int item) {
			if (Info) Log.i(TAG, "onContactClick: onClick()");
			String url = me.getString(R.string.contact_uri);
			String url2 = url;
			final Intent intent = new Intent();
			intent.setAction(Intent.ACTION_VIEW);
			intent.setData(Uri.parse(url2));
			me.startActivity(intent);
		}
	};
	

}
