// ◇ UTF-8
/**
 * メインビュー
 * MyGL2Viewを継承して作成
 */

package net.gorry.example.detachbug;

import net.gorry.mygl2.GLPoly4Tex;
import net.gorry.mygl2.MyGL2View;
import net.gorry.mygl2.ShaderProgram;

import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.opengl.GLES20;
import android.opengl.GLUtils;
import android.os.Bundle;
import android.util.Log;

/**
 * メインビュー（GLPJActivityから継承）
 *
 */
public class MainView extends MyGL2View {
	private static final String TAG="StencilBug";
	private static final String SUBTAG="MainView";
	private static final boolean Verbose = false;
	private static final boolean Debug = false;
	private static final boolean Info = false;
	// if (Verbose) Log.v(TAG, SUBTAG+": "+"");
	// if (Debug) Log.d(TAG, SUBTAG+": "+"");
	// if (Info) Log.i(TAG, SUBTAG+": "+"");

	@SuppressWarnings("unused")
	private ActivityMain me;

	/**
	 * コンストラクタ
	 * @param activity アクティビティ
	 * @param r Redビット数
	 * @param g Greenビット数
	 * @param b Blueビット数
	 * @param a Alphaビット数
	 * @param depth depthビット数
	 * @param stencil stencilビット数
	 */
	public MainView(ActivityMain activity, int r, int g, int b, int a, int depth, int stencil) {
		super(activity, r, g, b, a, depth, stencil);
		if (Info) Log.i(TAG, SUBTAG+": "+"constructor(): start");
		me = activity;
		if (Info) Log.i(TAG, SUBTAG+": "+"constructor(): end");
	}

	/**
	 * onPause
	 */
	@Override
	public void onPause() {
		if (Info) Log.i(TAG, SUBTAG+": "+"onPause(): start");

		super.onPause();

		if (Info) Log.i(TAG, SUBTAG+": "+"onPause(): end");
	}

	/**
	 * onOpenWindow
	 */
	@Override
	public void onOpenWindow(int width, int height) {
		if (Info) Log.i(TAG, SUBTAG+": "+"onOpenWindow(): start");

		super.onOpenWindow(width, height);
		setupGraphics(width, height);

		if (Info) Log.i(TAG, SUBTAG+": "+"onOpenWindow(): end");
	}

	/**
	 * onCloseWindow
	 */
	@Override
	public void onCloseWindow() {
		if (Info) Log.i(TAG, SUBTAG+": "+"onCloseWindow(): start");

		super.onCloseWindow();
		releaseGraphics();

		if (Info) Log.i(TAG, SUBTAG+": "+"onCloseWindow(): end");
	}

	/**
	 * onFrame
	 */
	@Override
	public void onFrame() {
		boolean ret = true;
		if (Debug) Log.d(TAG, SUBTAG+": "+"onFrame(): start");

		super.onFrame();
		setObjects();
		renderFrame();

		if (Debug) Log.d(TAG, SUBTAG+": "+"onFrame(): end: ret="+ret);
	}

	// =================================================================

	private static final String Poly4TexWhite_fsh =

"	precision mediump float;						\n"+
"	varying vec2 v_texCoord;						\n"+
"	uniform sampler2D u_texUnitId;					\n"+
"													\n"+
"	void main()										\n"+
"	{												\n"+
"		vec4 tmp;									\n"+
"		tmp = texture2D(u_texUnitId, v_texCoord);	\n"+
"		if (tmp.a >= 1.0/255.0) {					\n"+
"			tmp.rgb = vec3(1.0,1.0,1.0);			\n"+
"		}											\n"+
"		gl_FragColor = tmp;							\n"+
"	}												\n"+
"";

	private static final String Poly4Tex_fsh =

"	precision mediump float;						\n"+
"	varying vec2 v_texCoord;						\n"+
"	uniform sampler2D u_texUnitId;					\n"+
"													\n"+
"	void main()										\n"+
"	{												\n"+
"		vec4 tmp;									\n"+
"		tmp = texture2D(u_texUnitId, v_texCoord);	\n"+
"		gl_FragColor = tmp;							\n"+
"	}												\n"+
"";

	private static final String Poly4Tex_vsh =

"	precision mediump float;							\n"+
"	attribute vec4 a_pos;								\n"+
"	attribute vec2 a_texCoord;							\n"+
"	uniform vec2 u_scrSize;								\n"+
"	uniform vec2 u_texSize;								\n"+
"	uniform mat4 u_mvpMat;								\n"+
"	varying vec2 v_texCoord;							\n"+
"														\n"+
"	void main()											\n"+
"	{													\n"+
"		vec4 tmp;										\n"+
"		tmp = u_mvpMat * a_pos;							\n"+
"		gl_Position.x = (tmp.x *  2.0) / u_scrSize.x;	\n"+
"		gl_Position.y = (tmp.y * -2.0) / u_scrSize.y;	\n"+
"		gl_Position.z = a_pos.z;						\n"+
"		gl_Position.w = a_pos.w;						\n"+
"		v_texCoord.x = a_texCoord.x / u_texSize.x;		\n"+
"		v_texCoord.y = a_texCoord.y / u_texSize.y;		\n"+
"	}													\n"+
"";

	public static int mDetachTestMode = 0;

	private int mFrameCount = 0;

	private static int OBJ_MAX = 3;
	private GLPoly4Tex[] mObj = new GLPoly4Tex[OBJ_MAX];
	private ShaderProgram mShaderProgram;
	private ShaderProgram mShaderProgramWhite;

	private void loadObj(int objno, int resno, ShaderProgram sp) {
		if (Info) Log.i(TAG, SUBTAG+": "+"loadObj(): start: objno="+objno+", resno="+resno);

		Bitmap bmp = BitmapFactory.decodeResource(me.getResources(), resno);
		mObj[objno] = new GLPoly4Tex(bmp, sp);
		if (mObj[objno] == null) {
			Log.e(TAG, SUBTAG+": "+"loadObj(): failed: new GLPoly4Tex()");
		}

		if (Info) Log.i(TAG, SUBTAG+": "+"loadObj(): end");
	}

	private void drawObj(int objno) {
		mObj[objno].draw();
	}

	private void unloadObj(int objno) {
		mObj[objno].dispose();
	}

	/**
	 * OBJセットアップ
	 * @param w スクリーン横幅
	 * @param h スクリーン縦幅
	 * @return 成功ならtrue
	 */
	public boolean setupGraphics(int w, int h) {
		if (Info) Log.i(TAG, SUBTAG+": "+"setupGraphics(): w="+w+", h="+h);

		int i;
		boolean ret;

		mShaderProgram = new ShaderProgram(w, h);
		ret = mShaderProgram.attachProgram(Poly4Tex_vsh, Poly4Tex_fsh);
		if (!ret) {
			Log.e(TAG, SUBTAG+": "+"setupGraphics(): failed: mShaderProgram");
			return false;
		}

		mShaderProgramWhite = new ShaderProgram(w, h);
		ret = mShaderProgramWhite.attachProgram(Poly4Tex_vsh, Poly4TexWhite_fsh);
		if (!ret) {
			Log.e(TAG, SUBTAG+": "+"setupGraphics(): failed: mShaderProgramWhite");
			return false;
		}

		// mShaderProgramWhiteに対してデタッチを行う
		switch (mDetachTestMode) {
		  case 1:
			// バーテックスシェーダをdetach
			GLES20.glDetachShader(mShaderProgramWhite.programId(), mShaderProgramWhite.vertexShaderId());
			break;

		  case 2:
			// フラグメントシェーダをdetach
			GLES20.glDetachShader(mShaderProgramWhite.programId(), mShaderProgramWhite.fragmentShaderId());
			break;

		  case 3:
			// バーテックス／フラグメントシェーダをdetach
			GLES20.glDetachShader(mShaderProgramWhite.programId(), mShaderProgramWhite.vertexShaderId());
			GLES20.glDetachShader(mShaderProgramWhite.programId(), mShaderProgramWhite.fragmentShaderId());
			break;
		}

		loadObj(0, R.drawable.square, mShaderProgramWhite);
		loadObj(1, R.drawable.star, mShaderProgramWhite);
		loadObj(2, R.drawable.circle, mShaderProgramWhite);

		if (Info) Log.i(TAG, SUBTAG+": "+"setupGraphics(): end");
	
		return true;
	}

	/**
	 * OBJリリース
	 * @return 成功ならtrue
	 */
	public boolean releaseGraphics() {
		if (Info) Log.i(TAG, SUBTAG+": "+"releaseGraphics(): start");

		unloadObj(0);
		unloadObj(1);
		unloadObj(2);

		if (mShaderProgram != null) {
			mShaderProgram.detachProgram();
		}
		
		if (mShaderProgramWhite != null) {
			mShaderProgramWhite.detachProgram();
		}
		
		if (Info) Log.i(TAG, SUBTAG+": "+"releaseGraphics(): end");
		return true;
	}


	/**
	 * オブジェクトの情報設定
	 * @return 成功ならtrue
	 */

	public boolean setObjects()
	{
		int i;

		if (Debug) Log.d(TAG, SUBTAG+": "+"setObjects(): start");

		mFrameCount++;

		mObj[0].mOffsetY = 0;
		mObj[0].mAngle = -(mFrameCount) % 360;

		mObj[1].mOffsetY = 192;
		mObj[1].mAngle = (mFrameCount) % 360;

		mObj[2].mOffsetY = -192;
		mObj[2].mAngle = (mFrameCount*4) % 360;
		
		if (Debug) Log.d(TAG, SUBTAG+": "+"setObjects(): end");
		return true;
	}

	/**
	 * フレームレンダリング
	 * @return 成功ならtrue
	 */
	public boolean renderFrame()
	{
		if (Debug) Log.d(TAG, SUBTAG+": "+"renderFrame(): start");
	
		// 背景塗りつぶし
		GLES20.glClearColor(0.0f, 0, 0, 0.0f);
		GLES20.glClear( GLES20.GL_DEPTH_BUFFER_BIT | GLES20.GL_COLOR_BUFFER_BIT);

		// 四角を描画
		drawObj(0);

		// 星を描画
		drawObj(1);

		// 丸を描画
		drawObj(2);

		if (Debug) Log.d(TAG, SUBTAG+": "+"renderFrame(): end");
		return true;
	}

}

