// ◇ UTF-8
/**
 * GLPoly4Tex
 * ４頂点テクスチャポリゴン
 */

package net.gorry.mygl2;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;
import java.nio.ShortBuffer;

import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.opengl.GLES20;
import android.opengl.GLUtils;
import android.util.Log;

public class GLPoly4Tex {
	private static final String TAG="MyGL2";
	private static final String SUBTAG="GLPoly4Tex";
	private static final boolean Verbose = false;
	private static final boolean Debug = false;
	private static final boolean Info = false;
	// if (Verbose) Log.v(TAG, SUBTAG+": "+"");
	// if (Debug) Log.d(TAG, SUBTAG+": "+"");
	// if (Info) Log.i(TAG, SUBTAG+": "+"");

	private static final int FLOAT_SIZE_BYTES = 4;
	private static final int SHORT_SIZE_BYTES = 2;

	private static float[] mMat = new float[16];
	private ShortBuffer mIndices;
	private int mVboIndices;
	private FloatBuffer mVertex;
	private int mVboVertex;

	// 頂点情報: (X, Y, Z, U, V)*4
	public static final int VERTSIZE = 5;
	public static final int VERT = 4;
	public static final int VERT_X = 0;
	public static final int VERT_Y = 1;
	public static final int VERT_Z = 2;
	public static final int VERT_U = 3;
	public static final int VERT_V = 4;

	public static final int INDICESSIZE = 6;
	private static final short mIndicesData[] = { 0, 1, 2, 0, 2, 3 };

	public float[] mVert = new float[VERTSIZE*VERT];
	public int mTexWidth;
	public int mTexHeight;
	public int mTexId;
	public ShaderProgram mShaderProgram;
	public float mMagX;
	public float mMagY;
	public float mAngle;
	public float mOffsetX;
	public float mOffsetY;
	public boolean mPremultiplied;

	/**
	 * コンストラクタ
	 */
	public GLPoly4Tex(Bitmap bmp, ShaderProgram sp) {
		if (Info) Log.i(TAG, SUBTAG+": "+"GLPoly4Tex(): start");

		int texId;
		int texWidth;
		int texHeight;
		int[] objTex = new int[1];
		GLES20.glGenTextures(1, objTex, 0);
		GLES20.glBindTexture(GLES20.GL_TEXTURE_2D, objTex[0]);
		GLUtils.texImage2D(GLES20.GL_TEXTURE_2D, 0, bmp, 0);
		texWidth = bmp.getWidth();
		texHeight = bmp.getHeight();
		bmp.recycle();
		if (Info) Log.i(TAG, SUBTAG+": "+"GLPoly4Tex(): texId="+objTex[0]+", texWidth="+texWidth+", texHeight="+texHeight);
		
		mTexWidth = texWidth;
		mTexHeight = texHeight;
		mTexId = objTex[0];
		mShaderProgram = sp;
		mMagX = 1.0f;
		mMagY = 1.0f;
		mAngle = 0.0f;
		mOffsetX = 0.0f;
		mOffsetY = 0.0f;
		mPremultiplied = true;

		// 左上
		mVert[VERTSIZE*0+VERT_X] = -(texWidth/2.0f);
		mVert[VERTSIZE*0+VERT_Y] = -(texHeight/2.0f);
		mVert[VERTSIZE*0+VERT_Z] = 0.0f;
		mVert[VERTSIZE*0+VERT_U] = 0;
		mVert[VERTSIZE*0+VERT_V] = 0;

		// 右上
		mVert[VERTSIZE*1+VERT_X] = +(texWidth/2.0f);
		mVert[VERTSIZE*1+VERT_Y] = -(texHeight/2.0f);
		mVert[VERTSIZE*1+VERT_Z] = 0.0f;
		mVert[VERTSIZE*1+VERT_U] = texWidth;
		mVert[VERTSIZE*1+VERT_V] = 0;

		// 右下
		mVert[VERTSIZE*2+VERT_X] = +(texWidth/2.0f);
		mVert[VERTSIZE*2+VERT_Y] = +(texHeight/2.0f);
		mVert[VERTSIZE*2+VERT_Z] = 0.0f;
		mVert[VERTSIZE*2+VERT_U] = texWidth;
		mVert[VERTSIZE*2+VERT_V] = texHeight;

		// 左下
		mVert[VERTSIZE*3+VERT_X] = -(texWidth/2.0f);
		mVert[VERTSIZE*3+VERT_Y] = +(texHeight/2.0f);
		mVert[VERTSIZE*3+VERT_Z] = 0.0f;
		mVert[VERTSIZE*3+VERT_U] = 0;
		mVert[VERTSIZE*3+VERT_V] = texHeight;

		// VBOバッファ作成
		int[] buf = new int[2];
		GLES20.glGenBuffers(2, buf, 0);
		mVboVertex = buf[0];
		mVboIndices = buf[1];

		// Indicesデータの作成
		{
			ByteBuffer byteBuffer = ByteBuffer.allocateDirect(INDICESSIZE*SHORT_SIZE_BYTES);
			byteBuffer.order(ByteOrder.nativeOrder());
			mIndices = byteBuffer.asShortBuffer();
			mIndices.put(mIndicesData);
			mIndices.position(0);
			GLES20.glBindBuffer(GLES20.GL_ELEMENT_ARRAY_BUFFER, mVboIndices);
			GLES20.glBufferData(
					GLES20.GL_ELEMENT_ARRAY_BUFFER, 
					INDICESSIZE*SHORT_SIZE_BYTES, mIndices,
					GLES20.GL_STATIC_DRAW
					);
		}

		// Vertexデータの作成
		{
			ByteBuffer byteBuffer = ByteBuffer.allocateDirect(VERTSIZE*VERT*FLOAT_SIZE_BYTES);
			byteBuffer.order(ByteOrder.nativeOrder());
			mVertex = byteBuffer.asFloatBuffer();
			mVertex.put(mVert);
			mVertex.position(0);
			GLES20.glBindBuffer(GLES20.GL_ARRAY_BUFFER, mVboVertex);
			GLES20.glBufferData(
					GLES20.GL_ARRAY_BUFFER, 
					VERTSIZE*VERT*FLOAT_SIZE_BYTES, mVertex, 
					GLES20.GL_STATIC_DRAW
					);
		}

		if (Info) Log.i(TAG, SUBTAG+": "+"GLPoly4Tex(): end");
	}

	// デストラクタ
	public void dispose() {
		if (mTexId > 0) {
			int[] objTex = new int[1];
			objTex[0] = mTexId; 
			GLES20.glDeleteTextures(1, objTex, 0);
		}
	}
	
	/**
	 * 描画
	 */
	public void draw() {
		if (Info) Log.i(TAG, SUBTAG+": "+"GLPoly4Tex(): start");

		ShaderProgram sp = mShaderProgram;

		// シェーダプログラムの設定
		sp.useProgram();

		// バッファの登録
		GLES20.glBindBuffer(GLES20.GL_ARRAY_BUFFER, mVboVertex);

		// 頂点の登録
		GLES20.glVertexAttribPointer( 
		  sp.mLoc_a_pos,
		  3, GLES20.GL_FLOAT, 
		  false, VERTSIZE*FLOAT_SIZE_BYTES, VERT_X*FLOAT_SIZE_BYTES
		);
		GLES20.glEnableVertexAttribArray(sp.mLoc_a_pos);

		// テクスチャの座標登録
		GLES20.glVertexAttribPointer( 
		  sp.mLoc_a_texCoord,
		  2, GLES20.GL_FLOAT, 
		  false, VERTSIZE*FLOAT_SIZE_BYTES, VERT_U*FLOAT_SIZE_BYTES
		);
		GLES20.glEnableVertexAttribArray(sp.mLoc_a_texCoord);

		// テクスチャの登録
		GLES20.glActiveTexture(GLES20.GL_TEXTURE0);
		GLES20.glBindTexture(GLES20.GL_TEXTURE_2D, mTexId);

		// テクスチャユニットの登録
		GLES20.glUniform1i(sp.mLoc_u_texUnitId, 0);

		// スクリーンサイズの登録
		GLES20.glUniform2f(sp.mLoc_u_scrSize, sp.mScreenWidth, sp.mScreenHeight);

		// テクスチャサイズの登録
		GLES20.glUniform2f(sp.mLoc_u_texSize, mTexWidth, mTexHeight);

		// 回転行列
		float angle = (float)(mAngle*Math.PI/180.0f);
		mMat[0*4+0] = (float)( Math.cos(angle)*mMagX);
		mMat[0*4+1] = (float)(-Math.sin(angle)*mMagX);
		mMat[0*4+2] = 0.0f;
		mMat[0*4+3] = 0.0f;
		mMat[1*4+0] = (float)( Math.sin(angle)*mMagY);
		mMat[1*4+1] = (float)( Math.cos(angle)*mMagY);
		mMat[1*4+2] = 0.0f;
		mMat[1*4+3] = 0.0f;
		mMat[2*4+0] = 0.0f;
		mMat[2*4+1] = 0.0f;
		mMat[2*4+2] = 0.0f;
		mMat[2*4+3] = 0.0f;
		mMat[3*4+0] = mOffsetX;
		mMat[3*4+1] = mOffsetY;
		GLES20.glUniformMatrix4fv(sp.mLoc_u_mvpMat, 1, false, mMat, 0);

		// 拡大/縮小フィルタの設定
		GLES20.glTexParameteri(GLES20.GL_TEXTURE_2D, GLES20.GL_TEXTURE_MIN_FILTER, GLES20.GL_LINEAR);
		GLES20.glTexParameteri(GLES20.GL_TEXTURE_2D, GLES20.GL_TEXTURE_MAG_FILTER, GLES20.GL_LINEAR);

		// 拡大/縮小ラップの設定
		GLES20.glTexParameteri(GLES20.GL_TEXTURE_2D, GLES20.GL_TEXTURE_WRAP_S, GLES20.GL_CLAMP_TO_EDGE);
		GLES20.glTexParameteri(GLES20.GL_TEXTURE_2D, GLES20.GL_TEXTURE_WRAP_T, GLES20.GL_CLAMP_TO_EDGE);

		// ブレンド機能の設定
		try {
			GLES20.glBlendEquation(GLES20.GL_FUNC_ADD);
		} catch (Exception e) {
			// gingerbreadではGLES20.glBlendEquation()が動作しないものがあるらしい
		}
		if (mPremultiplied) {
			GLES20.glBlendFunc(GLES20.GL_ONE, GLES20.GL_ONE_MINUS_SRC_ALPHA);
		} else {
			GLES20.glBlendFunc(GLES20.GL_SRC_ALPHA, GLES20.GL_ONE_MINUS_SRC_ALPHA);
		}
		GLES20.glEnable(GLES20.GL_BLEND);
		
		// 描画開始
		GLES20.glBindBuffer(GLES20.GL_ELEMENT_ARRAY_BUFFER, mVboIndices);
		GLES20.glDrawElements(
		  GLES20.GL_TRIANGLES, 
		  INDICESSIZE,
		  GLES20.GL_UNSIGNED_SHORT,
		  0
		);

		if (Info) Log.i(TAG, SUBTAG+": "+"GLPoly4Tex(): end");

	}
}
