// ◇ UTF-8
/**
 * MyGL2View
 * GL2ビューのフレームワーク
 */

package net.gorry.mygl2;

import android.app.Activity;
import android.content.Context;
import android.content.res.AssetManager;
import android.graphics.PixelFormat;
import android.opengl.GLES20;
import android.opengl.GLSurfaceView;
import android.os.Bundle;
import android.util.AttributeSet;
import android.util.Log;
import android.view.KeyEvent;
import android.view.MotionEvent;

import javax.microedition.khronos.egl.EGL10;
import javax.microedition.khronos.egl.EGLConfig;
import javax.microedition.khronos.egl.EGLContext;
import javax.microedition.khronos.egl.EGLDisplay;
import javax.microedition.khronos.opengles.GL10;

/**
 * @author gorry
 *
 */
public class MyGL2View extends GLSurfaceView {
	private static final String TAG="MyGL2";
	private static final String SUBTAG="MyGL2View";
	private static final boolean Verbose = false;
	private static final boolean Debug = false;
	private static final boolean Info = true; //false;
	// if (Verbose) Log.v(TAG, SUBTAG+": "+"");
	// if (Debug) Log.d(TAG, SUBTAG+": "+"");
	// if (Info) Log.i(TAG, SUBTAG+": "+"");

	private Activity me;

	private Renderer mRenderer;

	/**
	 * コンストラクタ
	 * @param activity アクティビティ
	 * @param r Redビット数
	 * @param g Greenビット数
	 * @param b Blueビット数
	 * @param a Alphaビット数
	 * @param depth depthビット数
	 * @param stencil stencilビット数
	 */
	public MyGL2View(Activity activity, int r, int g, int b, int a, int depth, int stencil) {
		super(activity);
		if (Info) Log.i(TAG, SUBTAG+": "+"construct(): start");
		init(r, g, b, a, depth, stencil);
		if (Info) Log.i(TAG, SUBTAG+": "+"construct(): end");
	}

	/**
	 * onPause
	 */
	@Override
	public void onPause() {
		if (Info) Log.i(TAG, SUBTAG+": "+"onPause(): start");

		queueEvent(new Runnable(){
			public void run() {
				mRenderer.onCloseWindow();
			}
		});

		super.onPause();
		if (Info) Log.i(TAG, SUBTAG+": "+"onPause(): end");
	}

	/**
	 * onResume
	 */
	@Override
	public void onResume() {
		if (Info) Log.i(TAG, SUBTAG+": "+"onResume(): start");

		super.onResume();
		if (Info) Log.i(TAG, SUBTAG+": "+"onResume(): end");
	}

	/**
	 * onOpenWindow
	 */
	public void onOpenWindow(int width, int height) {
		if (Info) Log.i(TAG, SUBTAG+": "+"onOpenWindow(): start");

		GLES20.glViewport(0, 0, width, height);

		if (Info) Log.i(TAG, SUBTAG+": "+"onOpenWindow(): end");
	}

	/**
	 * onCloseWindow
	 */
	public void onCloseWindow() {
		boolean ret = true;
		if (Info) Log.i(TAG, SUBTAG+": "+"onCloseWindow(): start");

		if (Info) Log.i(TAG, SUBTAG+": "+"onCloseWindow(): end");
	}

	/**
	 * onFrame
	 */
	public void onFrame() {
		if (Debug) Log.d(TAG, SUBTAG+": "+"onFrame(): start");

		if (Debug) Log.d(TAG, SUBTAG+": "+"onFrame(): end");
	}

	/**
	 * 初期化
	 */
	private void init(int r, int g, int b, int a, int depth, int stencil) {
		if (Info) Log.i(TAG, SUBTAG+": "+"init(): start: "+String.format("r,g,b,a=%d,%d,%d,%d, depth=%d, stencil=%d", r,g,b,a, depth, stencil));

		/* By default, GLSurfaceView() creates a RGB_565 opaque surface.
		 * If we want a translucent one, we should change the surface's
		 * format here, using PixelFormat.TRANSLUCENT for GL Surfaces
		 * is interpreted as any 32-bit surface with alpha by SurfaceFlinger.
		 */
		if (a > 0) {
			this.getHolder().setFormat(PixelFormat.TRANSLUCENT);
		}

		/* Setup the context factory for 2.0 rendering.
		 * See ContextFactory class definition below
		 */
		setEGLContextFactory(new ContextFactory());

		/* We need to choose an EGLConfig that matches the format of
		 * our surface exactly. This is going to be done in our
		 * custom config chooser. See ConfigChooser class definition
		 * below.
		 */
		setEGLConfigChooser(
			 new ConfigChooser(r, g, b, a, depth, stencil)
		);

		/* Set the renderer responsible for frame rendering */
		mRenderer = new Renderer(this);
		setRenderer(mRenderer);

		if (Info) Log.i(TAG, SUBTAG+": "+"init(): end");
	}

	private static class ContextFactory implements GLSurfaceView.EGLContextFactory {
		private static int EGL_CONTEXT_CLIENT_VERSION = 0x3098;
		public EGLContext createContext(EGL10 egl, EGLDisplay display, EGLConfig eglConfig) {
			if (Info) Log.i(TAG, SUBTAG+": "+"createContext(): start");

			checkEglError("Before eglCreateContext", egl);
			int[] attrib_list = {EGL_CONTEXT_CLIENT_VERSION, 2, EGL10.EGL_NONE };
			EGLContext context = egl.eglCreateContext(display, eglConfig, EGL10.EGL_NO_CONTEXT, attrib_list);
			checkEglError("After eglCreateContext", egl);

			if (Info) Log.i(TAG, SUBTAG+": "+"createContext(): end");
			return context;
		}

		public void destroyContext(EGL10 egl, EGLDisplay display, EGLContext context) {
			if (Info) Log.i(TAG, SUBTAG+": "+"destroyContext(): start");

			egl.eglDestroyContext(display, context);

			if (Info) Log.i(TAG, SUBTAG+": "+"destroyContext(): end");
		}
	}

	private static void checkEglError(String prompt, EGL10 egl) {
		int error;
		while ((error = egl.eglGetError()) != EGL10.EGL_SUCCESS) {
			Log.e(TAG, SUBTAG+": "+"checkEglError(): "+String.format("%s: EGL error: 0x%x", prompt, error));
		}
	}

	private static class ConfigChooser implements GLSurfaceView.EGLConfigChooser {

		public ConfigChooser(int r, int g, int b, int a, int depth, int stencil) {
			if (Info) Log.i(TAG, SUBTAG+": "+"ConfigChooser(): "+String.format("r=%d, g=%d, b=%d, a=%d, depth=%d, stencil=%d", r,g,b,a, depth, stencil));
			mRedSize = r;
			mGreenSize = g;
			mBlueSize = b;
			mAlphaSize = a;
			mDepthSize = depth;
			mStencilSize = stencil;
		}

		/* This EGL config specification is used to specify 2.0 rendering.
		 * We use a minimum size of 4 bits for red/green/blue, but will
		 * perform actual matching in chooseConfig() below.
		 */
		private static int EGL_OPENGL_ES2_BIT = 4;
		private static int[] s_configAttribs2 =
		{
			EGL10.EGL_RED_SIZE, 4,
			EGL10.EGL_GREEN_SIZE, 4,
			EGL10.EGL_BLUE_SIZE, 4,
			EGL10.EGL_RENDERABLE_TYPE, EGL_OPENGL_ES2_BIT,
			EGL10.EGL_NONE
		};

		public EGLConfig chooseConfig(EGL10 egl, EGLDisplay display) {
			if (Info) Log.i(TAG, SUBTAG+": "+"chooseConfig(): start");

			/* Get the number of minimally matching EGL configurations
			 */
			int[] num_config = new int[1];
			egl.eglChooseConfig(display, s_configAttribs2, null, 0, num_config);

			int numConfigs = num_config[0];

			if (numConfigs <= 0) {
				throw new IllegalArgumentException("No configs match configSpec");
			}

			/* Allocate then read the array of minimally matching EGL configs
			 */
			EGLConfig[] configs = new EGLConfig[numConfigs];
			egl.eglChooseConfig(display, s_configAttribs2, configs, numConfigs, num_config);

			if (Info) {
				 printConfigs(egl, display, configs);
			}
			/* Now return the "best" one
			 */
			EGLConfig ret = chooseConfig(egl, display, configs);
			if (Info) Log.i(TAG, SUBTAG+": "+"chooseConfig(): end");
			return ret;
		}

		public EGLConfig chooseConfig(EGL10 egl, EGLDisplay display, EGLConfig[] configs) {
			if (Info) Log.i(TAG, SUBTAG+": "+"chooseConfig(): start");

			for(EGLConfig config : configs) {
				int id = findConfigAttrib(egl, display, config, EGL10.EGL_CONFIG_ID, 0);
				int d = findConfigAttrib(egl, display, config, EGL10.EGL_DEPTH_SIZE, 0);
				int s = findConfigAttrib(egl, display, config, EGL10.EGL_STENCIL_SIZE, 0);
				int r = findConfigAttrib(egl, display, config, EGL10.EGL_RED_SIZE, 0);
				int g = findConfigAttrib(egl, display, config, EGL10.EGL_GREEN_SIZE, 0);
				int b = findConfigAttrib(egl, display, config, EGL10.EGL_BLUE_SIZE, 0);
				int a = findConfigAttrib(egl, display, config, EGL10.EGL_ALPHA_SIZE, 0);
				if (Info) Log.i(TAG, SUBTAG+": "+"chooseConfig(): "+String.format("id=%d, rgba=%d,%d,%d,%d, depth=%d, stencil=%d", id, r,g,b,a, d, s));

				// We need at least mDepthSize and mStencilSize bits
				if ((d != mDepthSize) || (s != mStencilSize)) {
					continue;
				}

				// We want an *exact* match for red/green/blue/alpha
				if ((r != mRedSize) || (g != mGreenSize) || (b != mBlueSize) || (a != mAlphaSize)) {
					continue;
				}

				Log.w(TAG, SUBTAG+": "+"chooseConfig(): end: matched: "+String.format("id=%d, rgba=%d,%d,%d,%d, depth=%d, stencil=%d", id, r,g,b,a, d, s));
				return config;
			}

			for(EGLConfig config : configs) {
				int id = findConfigAttrib(egl, display, config, EGL10.EGL_CONFIG_ID, 0);
				int d = findConfigAttrib(egl, display, config, EGL10.EGL_DEPTH_SIZE, 0);
				int s = findConfigAttrib(egl, display, config, EGL10.EGL_STENCIL_SIZE, 0);
				int r = findConfigAttrib(egl, display, config, EGL10.EGL_RED_SIZE, 0);
				int g = findConfigAttrib(egl, display, config, EGL10.EGL_GREEN_SIZE, 0);
				int b = findConfigAttrib(egl, display, config, EGL10.EGL_BLUE_SIZE, 0);
				int a = findConfigAttrib(egl, display, config, EGL10.EGL_ALPHA_SIZE, 0);
				if (Info) Log.i(TAG, SUBTAG+": "+"chooseConfig(): "+String.format("id=%d, rgba=%d,%d,%d,%d, depth=%d, stencil=%d", id, r,g,b,a, d, s));

				// We need at least mDepthSize and mStencilSize bits
				if ((d < mDepthSize) || (s < mStencilSize)) {
					continue;
				}

				// We want an *exact* match for red/green/blue/alpha
				if ((r < mRedSize) || (g < mGreenSize) || (b < mBlueSize) || (a < mAlphaSize)) {
					continue;
				}

				Log.w(TAG, SUBTAG+": "+"chooseConfig(): end: near matched: "+String.format("id=%d, rgba=%d,%d,%d,%d, depth=%d, stencil=%d", id, r,g,b,a, d, s));
				return config;
			}

			if (Info) Log.i(TAG, SUBTAG+": "+"chooseConfig(): failed: not matched");
			return null;
		}

		private int findConfigAttrib(EGL10 egl, EGLDisplay display,	EGLConfig config, int attribute, int defaultValue) {
			if (egl.eglGetConfigAttrib(display, config, attribute, mValue)) {
				return mValue[0];
			}
			return defaultValue;
		}

		private void printConfigs(EGL10 egl, EGLDisplay display, EGLConfig[] configs) {
			int numConfigs = configs.length;
			if (Info) Log.i(TAG, SUBTAG+": printConfigs(): "+String.format("%d configurations", numConfigs));
			for (int i = 0; i < numConfigs; i++) {
				if (Info) Log.i(TAG, SUBTAG+": printConfigs(): "+String.format("Configuration %d:\n", i));
				printConfig(egl, display, configs[i]);
			}
		}

		private void printConfig(EGL10 egl, EGLDisplay display,	EGLConfig config) {
			int[] attributes = {
					EGL10.EGL_BUFFER_SIZE,
					EGL10.EGL_ALPHA_SIZE,
					EGL10.EGL_BLUE_SIZE,
					EGL10.EGL_GREEN_SIZE,
					EGL10.EGL_RED_SIZE,
					EGL10.EGL_DEPTH_SIZE,
					EGL10.EGL_STENCIL_SIZE,
					EGL10.EGL_CONFIG_CAVEAT,
					EGL10.EGL_CONFIG_ID,
					EGL10.EGL_LEVEL,
					EGL10.EGL_MAX_PBUFFER_HEIGHT,
					EGL10.EGL_MAX_PBUFFER_PIXELS,
					EGL10.EGL_MAX_PBUFFER_WIDTH,
					EGL10.EGL_NATIVE_RENDERABLE,
					EGL10.EGL_NATIVE_VISUAL_ID,
					EGL10.EGL_NATIVE_VISUAL_TYPE,
					0x3030, // EGL10.EGL_PRESERVED_RESOURCES,
					EGL10.EGL_SAMPLES,
					EGL10.EGL_SAMPLE_BUFFERS,
					EGL10.EGL_SURFACE_TYPE,
					EGL10.EGL_TRANSPARENT_TYPE,
					EGL10.EGL_TRANSPARENT_RED_VALUE,
					EGL10.EGL_TRANSPARENT_GREEN_VALUE,
					EGL10.EGL_TRANSPARENT_BLUE_VALUE,
					0x3039, // EGL10.EGL_BIND_TO_TEXTURE_RGB,
					0x303A, // EGL10.EGL_BIND_TO_TEXTURE_RGBA,
					0x303B, // EGL10.EGL_MIN_SWAP_INTERVAL,
					0x303C, // EGL10.EGL_MAX_SWAP_INTERVAL,
					EGL10.EGL_LUMINANCE_SIZE,
					EGL10.EGL_ALPHA_MASK_SIZE,
					EGL10.EGL_COLOR_BUFFER_TYPE,
					EGL10.EGL_RENDERABLE_TYPE,
					0x3042 // EGL10.EGL_CONFORMANT
			};
			String[] names = {
					"EGL_BUFFER_SIZE",
					"EGL_ALPHA_SIZE",
					"EGL_BLUE_SIZE",
					"EGL_GREEN_SIZE",
					"EGL_RED_SIZE",
					"EGL_DEPTH_SIZE",
					"EGL_STENCIL_SIZE",
					"EGL_CONFIG_CAVEAT",
					"EGL_CONFIG_ID",
					"EGL_LEVEL",
					"EGL_MAX_PBUFFER_HEIGHT",
					"EGL_MAX_PBUFFER_PIXELS",
					"EGL_MAX_PBUFFER_WIDTH",
					"EGL_NATIVE_RENDERABLE",
					"EGL_NATIVE_VISUAL_ID",
					"EGL_NATIVE_VISUAL_TYPE",
					"EGL_PRESERVED_RESOURCES",
					"EGL_SAMPLES",
					"EGL_SAMPLE_BUFFERS",
					"EGL_SURFACE_TYPE",
					"EGL_TRANSPARENT_TYPE",
					"EGL_TRANSPARENT_RED_VALUE",
					"EGL_TRANSPARENT_GREEN_VALUE",
					"EGL_TRANSPARENT_BLUE_VALUE",
					"EGL_BIND_TO_TEXTURE_RGB",
					"EGL_BIND_TO_TEXTURE_RGBA",
					"EGL_MIN_SWAP_INTERVAL",
					"EGL_MAX_SWAP_INTERVAL",
					"EGL_LUMINANCE_SIZE",
					"EGL_ALPHA_MASK_SIZE",
					"EGL_COLOR_BUFFER_TYPE",
					"EGL_RENDERABLE_TYPE",
					"EGL_CONFORMANT"
			};
			int[] value = new int[1];
			if (Debug) {
				for (int i = 0; i < attributes.length; i++) {
					int attribute = attributes[i];
					String name = names[i];
					if ( egl.eglGetConfigAttrib(display, config, attribute, value)) {
						if (Debug) Log.d(TAG, SUBTAG+": printConfigs(): "+String.format("  %s: %d\n", name, value[0]));
					} else {
						// Log.w(TAG, String.format("  %s: failed\n", name));
						while (egl.eglGetError() != EGL10.EGL_SUCCESS) {
							// Do nothing.
						}
					}
				}
			} else {
				int d = findConfigAttrib(egl, display, config, EGL10.EGL_DEPTH_SIZE, 0);
				int s = findConfigAttrib(egl, display, config, EGL10.EGL_STENCIL_SIZE, 0);
				int r = findConfigAttrib(egl, display, config, EGL10.EGL_RED_SIZE, 0);
				int g = findConfigAttrib(egl, display, config, EGL10.EGL_GREEN_SIZE, 0);
				int b = findConfigAttrib(egl, display, config, EGL10.EGL_BLUE_SIZE, 0);
				int a = findConfigAttrib(egl, display, config, EGL10.EGL_ALPHA_SIZE, 0);
				if (Info) Log.i(TAG, SUBTAG+": printConfigs(): "+String.format("rgba=(%d,%d,%d,%d), depth=%d, stencil=%d", r,g,b,a, d, s));
			}
		}

		// Subclasses can adjust these values:
		protected int mRedSize;
		protected int mGreenSize;
		protected int mBlueSize;
		protected int mAlphaSize;
		protected int mDepthSize;
		protected int mStencilSize;
		private int[] mValue = new int[1];
	}

	private class Renderer implements GLSurfaceView.Renderer {
		private MyGL2View mView;
		/**
		 * コンストラクタ
		 */
		Renderer(MyGL2View v) {
			mView = v;
		}

		/**
		 * 画面更新
		 */
		public void onDrawFrame(GL10 gl) {
			if (Debug) Log.d(TAG, SUBTAG+": "+"onDrawFrame(): start");

			mView.onFrame();

			if (Debug) Log.d(TAG, SUBTAG+": "+"onDrawFrame(): end");
		}

		/**
		 * サーフェス状態変更
		 */
		public void onSurfaceChanged(GL10 gl, int width, int height) {
			if (Info) Log.i(TAG, SUBTAG+": "+"onSurfaceChanged(): start");

			mView.onOpenWindow(width, height);

			if (Info) Log.i(TAG, SUBTAG+": "+"onSurfaceChanged(): end");
		}

		public void onSurfaceCreated(GL10 gl, EGLConfig config) {
			if (Info) Log.i(TAG, SUBTAG+": "+"onSurfaceCreated(): start");
			// Do nothing.
			if (Info) Log.i(TAG, SUBTAG+": "+"onSurfaceCreated(): end");
		}
		
		public void onCloseWindow() {
			if (Info) Log.i(TAG, SUBTAG+": "+"onCloseWindow(): start");
			mView.onCloseWindow();
			if (Info) Log.i(TAG, SUBTAG+": "+"onCloseWindow(): end");
		}
	}

}
