// ◇ UTF-8
/**
 * ShaderProgram
 */

package net.gorry.mygl2;

import android.opengl.GLES20;
import android.util.Log;

/**
 * @author gorry
 *
 */
public class ShaderProgram {
	private static final String TAG="MyGL2";
	private static final String SUBTAG="ShaderProgram";
	private static final boolean Verbose = false;
	private static final boolean Debug = false;
	private static final boolean Info = false;
	// if (Verbose) Log.v(TAG, SUBTAG+": "+"");
	// if (Debug) Log.d(TAG, SUBTAG+": "+"");
	// if (Info) Log.i(TAG, SUBTAG+": "+"");

	private int mShaderProgram = 0;
	private Shader mVertexShader;
	private Shader mFragmentShader;

	/** 画面横幅 */
	public int mScreenWidth = 0;

	/** 画面縦幅 */
	public int mScreenHeight = 0;

	/** 頂点 */
	public int mLoc_a_pos = 0;

	/** テクスチャ座標 */
	public int mLoc_a_texCoord = 0;

	/** スクリーンサイズ */
	public int mLoc_u_scrSize = 0;

	/** テクスチャサイズ */
	public int mLoc_u_texSize = 0;

	/** 変形用行列 */
	public int mLoc_u_mvpMat = 0;

	/** テクスチャユニットID */
	public int mLoc_u_texUnitId = 0;

	/**
	 * コンストラクタ
	 * @param width 画面横幅
	 * @param height 画面縦幅
	 */
	public ShaderProgram(int width, int height) {
		if (Info) Log.i(TAG, SUBTAG+": "+"ShaderProgram(): start");

		mScreenWidth = width;
		mScreenHeight = height;

		if (Info) Log.i(TAG, SUBTAG+": "+"ShaderProgram(): end");
	}

	private class Shader {
		private int mId = 0;
		
		/**
		 * シェーダの読み込み
		 */
		public Shader(final int shaderType, final String source) {
			if (Info) Log.i(TAG, SUBTAG+": "+"loadShader(): start");

			// シェーダIDの取得
			mId = GLES20.glCreateShader(shaderType);
			if (mId == 0) {
				Log.e(TAG, SUBTAG+": "+"loadShader(): failed: shader == 0");
			}

			// シェーダのコンパイル
			GLES20.glShaderSource(mId, source);
			GLES20.glCompileShader(mId);
			int compiled[] = new int[1];
			GLES20.glGetShaderiv(mId, GLES20.GL_COMPILE_STATUS, compiled, 0);

			// コンパイルエラーの表示
			if (compiled[0] == 0) {
				int infoLen[] = new int[1];
				GLES20.glGetShaderiv(mId, GLES20.GL_INFO_LOG_LENGTH, infoLen, 0);
				if (infoLen[0] > 0) {
					String errstr = GLES20.glGetShaderInfoLog(mId);
					Log.e(TAG, SUBTAG+": "+String.format("loadShader(): Could not compile shader %d:\n%s\n", shaderType, errstr));
				}
				GLES20.glDeleteShader(mId);
				Log.e(TAG, SUBTAG+": "+"loadShader(): failed: compiled == 0");
				mId = 0;
			}

			if (Info) Log.i(TAG, SUBTAG+": "+"loadShader(): end: shader="+mId);
		}

		/**
		 * シェーダの解放
		 */
		public void unload() {
			if (Info) Log.i(TAG, SUBTAG+": "+"unloadShader(): start");

			if (mId != 0) {
				GLES20.glDeleteShader(mId);
			}

			if (Info) Log.i(TAG, SUBTAG+": "+"unloadShader(): end");
		}
	}

	/**
	 * シェーダプログラムの読み込み
	 */
	private int loadProgram(final String vertexSource, final String fragmentSource) {
		if (Info) Log.i(TAG, SUBTAG+": "+"loadProgram(): start");

		// バーテックスシェーダの読み込み
		mVertexShader = new Shader(GLES20.GL_VERTEX_SHADER, vertexSource);
		if (mVertexShader == null) {
			Log.e(TAG, SUBTAG+": "+"loadProgram(): failed: loadShader(GL_VERTEX_SHADER)");
			return 0;
		}

		// フラグメントシェーダの読み込み
		mFragmentShader = new Shader(GLES20.GL_FRAGMENT_SHADER, fragmentSource);
		if (mFragmentShader == null) {
			Log.e(TAG, SUBTAG+": "+"loadProgram(): failed: loadShader(GL_FRAGMENT_SHADER)");
			mVertexShader.unload();
			return 0;
		}

		// シェーダプログラムIDの取得
		mShaderProgram = GLES20.glCreateProgram();
		if (mShaderProgram == 0) {
			Log.e(TAG, SUBTAG+": "+"loadProgram(): failed: program == 0");
			mFragmentShader.unload();
			mVertexShader.unload();
			return 0;
		}

		// シェーダプログラムのリンク
		GLES20.glAttachShader(mShaderProgram, mVertexShader.mId);
		GLES20.glAttachShader(mShaderProgram, mFragmentShader.mId);
		GLES20.glLinkProgram(mShaderProgram);
		int linkStatus[] = new int[1];
		GLES20.glGetProgramiv(mShaderProgram, GLES20.GL_LINK_STATUS, linkStatus, 0);

		// リンクエラーの表示
		if (linkStatus[0] != GLES20.GL_TRUE) {
			int bufLength[] = new int[1];
			GLES20.glGetProgramiv(mShaderProgram, GLES20.GL_INFO_LOG_LENGTH, bufLength, 0);
			if (bufLength[0] > 0) {
				String errstr = GLES20.glGetProgramInfoLog(mShaderProgram);
				Log.e(TAG, SUBTAG+": "+String.format("loaProgram(): Could not link program %d:\n%s\n", mShaderProgram, errstr));
			}
			Log.e(TAG, SUBTAG+": "+"loadProgram(): failed: linkStatus != GL_TRUE");
			GLES20.glDeleteProgram(mShaderProgram);
			mFragmentShader.unload();
			mVertexShader.unload();
			return 0;
		}

		if (Info) Log.i(TAG, SUBTAG+": "+"loadProgram(): end: program="+mShaderProgram);

		return mShaderProgram;
	}

	/**
	 * シェーダプログラムの解放
	 */
	private void unloadProgram(int program) {
		if (Info) Log.i(TAG, SUBTAG+": "+"unloadProgram(): start");
		
		GLES20.glDeleteProgram(program);
		mFragmentShader.unload();
		mVertexShader.unload();

		if (Info) Log.i(TAG, SUBTAG+": "+"unloadProgram(): end");
}

	/**
	 * シェーダプログラムの割り当て
	 * @param vsh バーテックスシェーダ
	 * @param fsh フラグメントシェーダ
	 * @return 成功ならtrue
	 */
	public boolean attachProgram(String vsh, String fsh) {
		if (Info) Log.i(TAG, SUBTAG+": "+"attachProgram(): start");

		detachProgram();

		// シェーダプログラムの読み込み
		mShaderProgram = loadProgram(vsh, fsh);
		if (mShaderProgram == 0) {
			Log.e(TAG, SUBTAG+": "+"failed: loadProgram()");
			return false;
		}

		// シェーダ引数のピックアップ
		mLoc_a_pos = GLES20.glGetAttribLocation(mShaderProgram, "a_pos");
		mLoc_a_texCoord = GLES20.glGetAttribLocation(mShaderProgram, "a_texCoord");
		mLoc_u_scrSize = GLES20.glGetUniformLocation(mShaderProgram, "u_scrSize");
		mLoc_u_texSize = GLES20.glGetUniformLocation(mShaderProgram, "u_texSize");
		mLoc_u_mvpMat = GLES20.glGetUniformLocation(mShaderProgram, "u_mvpMat");
		mLoc_u_texUnitId = GLES20.glGetUniformLocation(mShaderProgram, "u_texUnitId");

		if (Debug) Log.d(TAG, SUBTAG+": "+"a_pos="+mLoc_a_pos);
		if (Debug) Log.d(TAG, SUBTAG+": "+"a_texCoord="+mLoc_a_texCoord);
		if (Debug) Log.d(TAG, SUBTAG+": "+"u_scrSize="+mLoc_u_scrSize);
		if (Debug) Log.d(TAG, SUBTAG+": "+"u_texSize="+mLoc_u_texSize);
		if (Debug) Log.d(TAG, SUBTAG+": "+"u_mvpMat="+mLoc_u_mvpMat);
		if (Debug) Log.d(TAG, SUBTAG+": "+"u_texUnitId="+mLoc_u_texUnitId);

		if (Info) Log.i(TAG, SUBTAG+": "+"attachProgram(): end");
		return true;
	}
	
	/**
	 * シェーダプログラムの解放
	 * @return 成功ならtrue
	 */
	public boolean detachProgram() {
		if (Info) Log.i(TAG, SUBTAG+": "+"detachProgram(): start");

		if (mShaderProgram != 0) {
			unloadProgram(mShaderProgram);
			mShaderProgram = 0;
		}

		if (Info) Log.i(TAG, SUBTAG+": "+"detachProgram(): end");
		return true;
	}

	/**
	 * シェーダプログラムの適用
	 */
	public void useProgram() {
		if (Info) Log.i(TAG, SUBTAG+": "+"useShaderProgram(): start");

		GLES20.glUseProgram(mShaderProgram);

		if (Info) Log.i(TAG, SUBTAG+": "+"useShaderProgram(): end");
	}

	public int vertexShaderId() {
		return mVertexShader.mId;
	}

	public int fragmentShaderId() {
		return mFragmentShader.mId;
	}

	public int programId() {
		return mShaderProgram;
	}

}
